// -*- C++ -*-
#include <iostream>
#include "ColorFull/Core/Trace_basis.h"
#include "ColorFull/Core/Orthogonal_basis.h"
#include "ColorFull/Core/Tree_level_gluon_basis.h"

using namespace ColorFull;
using namespace std;

// This is a little test program to get aquinted with ColorFull and its possibilities 

int main(){

	// Create a Trace_basis basis with for example n_q(=n_qbar)=1
	// and 3 gluons
	Trace_basis MyBasis(1,3);

	// Keep only color structures appearing to lowest order in pure QCD
	// Trace_basis MyBasis(1,3,0);
	// Create a basis for 2 qqbar pairs and 4 gluons, keeping color structures
	// appearing up to 1 loop in pure QCD
	// Trace_basis MyBasis2(2,4,1);

	// Print the basis to cout
	cout << "Constructed the basis: \n" << MyBasis;

	// Print the basis to a file with standard filename
	// (see the basis_file_name in Col_basis)
	MyBasis.write_out_Col_basis();
	// Print the basis to a file with user set filename
	MyBasis.write_out_Col_basis("ColorResults/MyBasis");

	// Read in a basis
	//Trace_basis MyBasis2;
	//MyBasis2.read_in_Col_basis("ColorResults/MyBasis");
	//cout << "\nRead in basis for " << MyBasis2.n_q
	//		<< " qqbar-pair(s) and " << MyBasis2.n_g << " gluons.\n"<< endl;

	// Often one would like to calculate the scalar product matrix
	// By default, both a Polynomial and a numerical versions are
	// calculated and saved to the members P_spm and d_spm respectively.
	MyBasis.scalar_product_matrix();

	// Write out the scalar product matrix to a file with default filename
	MyBasis.write_out_P_spm();
	// ... or to a file with given name
	MyBasis.write_out_P_spm("ColorResults/MyBasisSPM");


	// For processes with many partons one may want to only calculate
	// the scalar product matrix numerically
	// MyBasis.scalar_product_matrix_num();

	// The basis vectors are color amplitudes, objects of class Col_amp.
	// For example the 0th basis vector is:
	Col_amp Ca0=MyBasis.at(0);


	// We may want to study the effect of gluon exchange and gluon emission
	// on a Col_amp or a Col_basis.
	// Functions for this is contained in the class Col_functions.
	Col_functions Col_fun;
	cout << "\nThe result of exchanging a gluon between parton 1 and 3 in:\n"
			<< Ca0 << "\nis:\n" << Col_fun.exchange_gluon(Ca0, 1, 3) << endl << endl;
	cout << "Radiating a new gluon, gluon 77, from parton 3 we get:\n"
			<< Col_fun.emit_gluon(Ca0 ,3, 77) << endl << endl;

	// Calculate the color correlator < c | T_i T_j | c >,
	// Polynomial Polycc=Col_fun.color_correlator( Ca0, 1, 3, 77 );


	// Calculate the color soft anomalous dimension matrix for exchange
	// between parton p1 and parton p2;
	int p1=1;
	int p2=3;
	Poly_matr Cg=MyBasis.color_gamma(p1,p2);
	// we may want to write it out
	//cout <<"Color soft anomalous dimension matrix for exchange between "
	//		<< p1 << " and " << p2 << ".\n" << Cg ;
	// or write it out to a file
	Cg.write_out_Poly_matr("ColorResults/MyCg");


  return 0;

}

